@@ -239,8 +239,8 @@ void _apply_lifting(
         const std::span<const std::int32_t> x_dofs = x_dofmap.links(cell);
         for (std::size_t i = 0; i < x_dofs.size(); ++i)
         {
-          dolfinx::common::impl::copy_N<3>(
-              std::next(x_g.begin(), 3 * x_dofs[i]),
+          std::copy_n(
+              std::next(x_g.begin(), 3 * x_dofs[i]),3,
               std::next(coordinate_dofs.begin(), 3 * i));
         }
         // Tabulate tensor
@@ -324,8 +324,8 @@ void _apply_lifting(
         std::vector<double> coordinate_dofs(3 * num_dofs_g);
         for (std::size_t i = 0; i < x_dofs.size(); ++i)
         {
-          dolfinx::common::impl::copy_N<3>(
-              std::next(x_g.begin(), 3 * x_dofs[i]),
+          std::copy_n(
+              std::next(x_g.begin(), 3 * x_dofs[i]),3,
               std::next(coordinate_dofs.begin(), 3 * i));
         }
 
